from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from inicie.cropimage import _
from inicie.cropimage.id import ID
from inicie.cropimage.id import IID
from persistent.dict import PersistentDict
from plone.app.z3cform.layout import wrap_form
from plone.registry.interfaces import IRegistry
from plone.z3cform.crud import crud
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.component import getUtility


class CropImageIDControlPanelForm(crud.CrudForm):

    update_schema = IID

    label = _(u'Crop Image ID')

    def get_items(self):
        registry = getUtility(IRegistry)
        ids = registry['inicie.cropimage.ids']
        data = []
        for i in ids:
            data.append(
                (
                    str(i),
                    ID(i)
                )
            )
        return data

    def add(self, data):
        """Add new ID data to inicie.cropimage.ids registry.

        :param data: ID data.
        :type data: dict
        """
        registry = getUtility(IRegistry)
        ids = registry['inicie.cropimage.ids']
        ids.append(data.values()[0])
        registry['inicie.cropimage.ids'] = ids

    def remove(self, (id, item)):
        """Delete ID data from inicie.cropimage.ids registry.

        :param id: Unique ID name.
        :type id: str

        :param item: inicie.cropimage.id.ID instance.
        :type id: object
        """
        registry = getUtility(IRegistry)
        ids = registry['inicie.cropimage.ids']
        ids = [i for i in ids if i != id]
        registry['inicie.cropimage.ids'] = ids


CropImageIDControlPanelView = wrap_form(
    CropImageIDControlPanelForm,
    index=ViewPageTemplateFile('templates/controlpanel.pt')
)


class CropImageView(BrowserView):

    template = ViewPageTemplateFile('templates/crop_image.pt')

    def __call__(self):
        context = aq_inner(self.context)
        form = self.request.form
        if form.get('form.button.Save', None) is not None:
            name = 'inicie.cropimage.{0}'.format(
                form.get('field')
            )
            anno = IAnnotations(context)
            if anno.get(name) is None:
                anno[name] = PersistentDict()
            anno[name].update(self.dimension(form))
        return self.template()

    def fields(self):
        context = aq_inner(self.context)
        return context.restrictedTraverse("image-fields")()

    def dimension(self, form):
        keys = ['x1', 'x2', 'y1', 'y2', 'w', 'h']
        res = {}
        for key in keys:
            res.update(
                {key: form[key]}
            )
        return {form.get('id-name'): res}

    def images(self):
        context = aq_inner(self.context)
        results = []
        for field in self.fields():
            full = context.getField(field).tag(context)
            item = {
                'full-image': '{0} id="cropbox" {1}'.format(full[:4], full[5:]),
                'select': self.select(),
                'field': field,
                'previews': self.previews(field),
            }
            results.append(item)
        return results

    def current_url(self):
        context_state = getMultiAdapter(
            (self.context, self.request),
            name=u'plone_context_state'
        )
        return context_state.current_page_url()

    def select(self):
        registry = getUtility(IRegistry)
        ids = registry['inicie.cropimage.ids']
        name = 'id-name'
        html = '<select name="{0}" id="{0}">'.format(name)
        for i in ids:
            html += '<option>{0}</option>'.format(i)
        html += '</select>'
        return html

    def previews(self, field):
        context = aq_inner(self.context)
        anno = IAnnotations(context)
        name = 'inicie.cropimage.{0}'.format(field)
        prev = anno.get(name)
        if prev is not None:
            prevs = []
            for key in prev.keys():
                html = '<article class="preview" id="{0}-{1}-preview">'.format(field, key)
                html += '<h1>{0}</h1>'.format(key)
                html += context.restrictedTraverse('cropped-image')(field, key)
                html += '</article>'
                prevs.append(html)
            return prevs
